import tensorflow as tf
import sys, os
import numpy as np

parentdpath1 = os.path.dirname(os.path.abspath(__file__))
parentdpath = parentdpath1[:-16] + 'applications'
path_list = [parentdpath + '/fmrcnn', parentdpath]
sys.path += path_list
print(sys.path)

import coco
from mrcnn import model
from mrcnn import visualize
from libs.nets import nets_factory as network
from libs.nets import pyramid_network as pyramid_network
from libs.visualization.pil_utils import cat_id_to_cls_name, draw_img, draw_bbox

tf.app.flags.DEFINE_string('data_dir', '/home/shangyw2/tmp',
                           """output result dir """)
tf.app.flags.DEFINE_string('inferce_data_dir', '/nfsdata/wangpeng_data/data/fmrcnndata/coco',
                           """inferce_data """)
tf.app.flags.DEFINE_string('checkpoint_dir', '/home/shangyw2/train/fmrcnn',
                           """Directory where to read training checkpoints """)
tf.app.flags.DEFINE_float(
    'weight_decay', 1e-4, 'The weight decay on the model weights.')
FLAGS = tf.app.flags.FLAGS


def main():

    with tf.Graph().as_default():

        # feed ops
        img = tf.placeholder(tf.float32, [None, None, None, 3])
        ih = tf.placeholder(tf.int32, None)
        iw = tf.placeholder(tf.int32, None)
        boxs = tf.placeholder(tf.float32, [None, 5])
        masks = tf.placeholder(tf.int32, [None, None, None])
        im_shape = tf.shape(img)
        image = tf.reshape(img, (im_shape[0], im_shape[1], im_shape[2], 3))


        #data prepare
        ckpt_dir = FLAGS.checkpoint_dir
        batch_size = 1
        data_direct = coco.CocoDataset()
        data_direct.load_coco(FLAGS.inferce_data_dir, "val")
        data_direct.prepare()
        config = coco.CocoConfig()
        data_generator = model.data_generator_new(data_direct, config, shuffle=True, batch_size=batch_size)

        ## network FMCNN
        logits, end_points, pyramid_map = network.get_network(FLAGS.network, image,
                                                              weight_decay=FLAGS.weight_decay, is_training=True)
        outputs = pyramid_network.build(end_points, im_shape[1], im_shape[2], pyramid_map,
                                        num_classes=81,
                                        base_anchors=9,
                                        is_training=True,
                                        gt_boxes=boxs, gt_masks=masks
                                        )

        #total_loss = outputs['total_loss']

        #inference is begining
        config = tf.ConfigProto()
        config.gpu_options.allow_growth = True
        with tf.Session(config=config) as sess:
            saver = tf.train.Saver()
            if os.path.exists(ckpt_dir):
                print("unet Restoring Variables from Checkpoint.")
                saver.restore(sess, tf.train.latest_checkpoint(ckpt_dir))
                # saver.restore(sess, ckpt_dir + '/model.ckpt-110000')
            else:
                raise ValueError("Checkpoint dir must be provided to restore model.")

            for step in xrange(100):
                # true data
                batched_inputs, batched_outputs = data_generator.next()
                images = batched_inputs[0]
                gt_boxes = batched_inputs[1][0]
                gt_masks = batched_inputs[2][0]
                feed_data = {
                    img: images,
                    ih: images.shape[0],
                    iw: images.shape[1],
                    boxs: gt_boxes,
                    masks: gt_masks
                }
                total_loss = outputs['total_loss']

                input_image = end_points['input']
                final_box = outputs['final_boxes']['box']
                final_cls = outputs['final_boxes']['cls']
                final_prob = outputs['final_boxes']['prob']
                final_gt_cls = outputs['final_boxes']['gt_cls']
                gt = outputs['gt']
                loss,image_np,box_np,class_np,pro_np,gt_cla_np,gt_np = sess.run([total_loss,input_image,final_box,final_cls,final_prob,final_gt_cls,gt],feed_data)

                #save result

                visualize.display_instances(image_np,box_np)
                print ("Verification error= {:.1f}%, loss= {:.4f}", loss)


if __name__ == '__main__':
    main()