# -----------------------------------------------------------
# Copyright (c) 2018 Lenovo AI Platform. All Rights Reserved.
# -----------------------------------------------------------

'''
Example:
# Infer
python unet_image.py --checkpoint_dir='/home/zhouzd2/train67' --output_dir=/home/zhouzd2/tmp --inferce_data_dir=/home/zhouzd2/letrain/applications/ehealth_unet/unet_image/training --input_fname_pattern=*.png
'''

import tensorflow as tf
import sys, os
parentdpath1 = os.path.dirname(os.path.abspath(__file__))
parentdpath = parentdpath1[:-14] + 'applications'
path_list = [parentdpath + '/ehealth_unet', parentdpath]
sys.path += path_list
print sys.path
import unet
import numpy as np
from util import combine_img_prediction, save_image
import os
import glob
import skimage.io as io
import skimage.transform as trans
import random

tf.app.flags.DEFINE_string('checkpoint_dir', '/home/zhouzd2/train67','checkpoint dir')
tf.app.flags.DEFINE_string('output_dir', '/home/zhouzd2/tmp','output result dir')
tf.app.flags.DEFINE_string('inferce_data_dir', '/nfsdata/wangpeng_data/data/unet_data/training','inferce data dir')
tf.app.flags.DEFINE_string("input_fname_pattern", "*.png", "Glob pattern of filename of input images [*]")

FLAGS = tf.app.flags.FLAGS

def DataGenerator(image_path, label_path,num_image,flag_multi_class = False,as_gray = True):
    for i in range(10):
        j = random.randint(0, len(num_image)-1)
        image_name = num_image[j].split('/').pop(-1)
        img = io.imread(os.path.join(image_path, image_name),as_gray = as_gray, as_grey = as_gray)
        #img = img / 255
        #img = trans.resize(img,target_size)
        img = np.reshape(img,img.shape+(1,)) if (not flag_multi_class) else img
        mask = io.imread(os.path.join(label_path, image_name), as_gray=as_gray, as_grey = as_gray)
        #mask = mask / 255
        #mask = trans.resize(mask, target_size)
        yield img, mask

def gen_data():
    data_path = FLAGS.inferce_data_dir
    # data_dir = '/home/zhouzd2/letrain/applications/ehealth_unet/unet/data/membrane/train'
    image_name_arr = glob.glob(os.path.join(data_path + '/image', FLAGS.input_fname_pattern))
    image_and_mask = DataGenerator(data_path + '/image', data_path + '/label', image_name_arr)
    return image_and_mask

def main():
    ckpt_dir = FLAGS.checkpoint_dir
    batch_size = 1
    save_image_dir = FLAGS.output_dir
    image_and_mask = gen_data()
    graph = tf.Graph().as_default()
    with graph:
        #batch_x, batch_y = batch_generator(batch_size, image_set)
        config = tf.ConfigProto()
        config.gpu_options.allow_growth = True
        with tf.Session(config=config) as sess:
            Input_channels = 1
            Nb_classes = 2

            net = unet.Unet(channels=Input_channels, n_class=Nb_classes, cost_kwargs=dict(class_weights=[1, 3]),
                            layers=5,
                            features_root=32)

            saver = tf.train.Saver()
            if os.path.exists(ckpt_dir):
                print("unet Restoring Variables from Checkpoint.")
                saver.restore(sess, tf.train.latest_checkpoint(ckpt_dir))
                #saver.restore(sess, ckpt_dir + '/model.ckpt-110000')
            else:
                print("unet Not able to find Checkpoint. Going to stop now...")
                raise
            for step in xrange(10):
                # load data
                batch_x, batch_y = [], []
                for i in xrange(batch_size):
                    x, y = image_and_mask.next()
                    batch_x.append(x)
                    batch_y.append(y)
                batch_x = np.array(batch_x, dtype=np.float32)
                batch_y = np.array(batch_y, dtype=np.uint8)
                loss, prediction = sess.run([net.cost, net.predicter], feed_dict={ net.x: batch_x,
                                                                                   net.y_specific: batch_y,
                                                                                   net.keep_prob: 1.})
                pred_shape = prediction.shape

                print ("Verification error= {:.1f}%, loss= {:.4f}".format(error_rate(prediction, batch_y), loss))
                img = combine_img_prediction(batch_x, batch_y, np.argmax(prediction, axis=-1))
                save_image(img, "%s/%s.jpg" % (save_image_dir, 'unet_num_%s_'%step))
            print ("************ inferce end ****************")


def error_rate(predictions, labels):
    """
    Return the error rate based on dense predictions and 1-hot labels.
    """

    return 100.0 - (
        100.0 *
        np.sum(np.argmax(predictions, 3) == labels) /
        (predictions.shape[0] * predictions.shape[1] * predictions.shape[2]))

if __name__ == '__main__':
    main()