#! /usr/bin/env python
# coding=utf-8

'''
Example:
# Infer
CUDA_VISIBLE_DEVICES=0  python letrain.py --model_name=yolov3 --inference=true --checkpoint_dir=/home/wangtao34/train/yolov3_2ps_2wr_sync --input_dir=/home/wangtao34/tmp/frcnn_input/demo/ --output_dir=/home/wangtao34/tmp/yolov3
'''

import cv2
import os
import sys
import numpy as np
import tensorflow as tf
from scipy.misc import imread, imsave, imshow, imresize


parentpath = os.path.dirname(os.path.abspath(__file__))
path_list = [parentpath[:-16] + 'applications',
             parentpath[:-16] + 'applications/' + 'yolov3']
sys.path += path_list
print(sys.path)

from core.yolov3 import YOLOV3
import core.utils as utils


FLAGS = tf.app.flags.FLAGS

return_elements = ["input/input_data:0", "pred_sbbox/concat_2:0", "pred_mbbox/concat_2:0", "pred_lbbox/concat_2:0"]
classes = utils.read_class_names(FLAGS.checkpoint_dir)
num_classes = len(classes)
input_size      = 416

def get_checkpoint():
    if tf.gfile.IsDirectory(FLAGS.checkpoint_dir):
        checkpoint_dir = tf.train.latest_checkpoint(FLAGS.checkpoint_dir)
    else:
        checkpoint_dir = FLAGS.checkpoint_dir

    return checkpoint_dir

def main():
    with tf.name_scope('input'):
        input_data = tf.placeholder(dtype=tf.float32, name='input_data')

    model = YOLOV3(input_data, trainable=False)
    print(model.conv_sbbox, model.conv_mbbox, model.conv_lbbox)
    _pred_sbbox, _pred_mbbox, _pred_lbbox = model.pred_sbbox, model.pred_mbbox, model.pred_lbbox


    saver = tf.train.Saver()
    with tf.Session(config=tf.ConfigProto(allow_soft_placement=True)) as sess:
        init = tf.global_variables_initializer()
        sess.run(init)

        saver.restore(sess, get_checkpoint())
        input_dir = FLAGS.input_dir
        if not input_dir.endswith('/'):
            input_dir = input_dir + '/'
        for dirpath, dirnames, filenames in os.walk(input_dir):
            if dirnames != []:
                print 'The test folder should contain only images, not folders'
                os._exit(0)
            for filename in filenames:
                p = input_dir + filename
                original_image = cv2.imread(p)
                original_image = cv2.cvtColor(original_image, cv2.COLOR_BGR2RGB)
                original_image_size = original_image.shape[:2]
                image_data = utils.image_preporcess(np.copy(original_image), [input_size, input_size])
                image_data = image_data[np.newaxis, ...]
                print('Demo for {}'.format(filename))
                pred_sbbox, pred_mbbox, pred_lbbox = sess.run(
                    [_pred_sbbox, _pred_mbbox, _pred_lbbox],
                            feed_dict={ input_data: image_data})

                pred_bbox = np.concatenate([np.reshape(pred_sbbox, (-1, 5 + num_classes)),
                                            np.reshape(pred_mbbox, (-1, 5 + num_classes)),
                                            np.reshape(pred_lbbox, (-1, 5 + num_classes))], axis=0)

                bboxes = utils.postprocess_boxes(pred_bbox, original_image_size, input_size, 0.3)
                bboxes = utils.nms(bboxes, 0.45, method='nms')
                image = utils.draw_bbox(original_image, bboxes)
                output_dir = FLAGS.output_dir
                if not tf.gfile.Exists(output_dir):
                    tf.gfile.MakeDirs(output_dir)
                filename_1, filename_2 = filename.split(".")
                ff = filename_1 + "_predict." + filename_2
                if not output_dir.endswith('/'):
                    output_dir = output_dir + '/'
                output_name = output_dir + ff
                imsave(output_name, image)
                print("Saved to {}".format(output_name))

                # image = Image.fromarray(image)
                # image.show()

if __name__ == '__main__':
  tf.logging.set_verbosity(tf.logging.INFO)
  tf.app.run()


